
#include <system_stm32f0xx.h>
#include <gpio.h>
#include <usart.h>
#include <tim.h>
#include <pwm.h>


/**
 * Program starts here.
 * @return
 */
void main(void)
{
    // Initialize system clock:  8MHz * PLLMULL6 = 48MHz
    SystemInit(USE_HSE, RCC_CFGR_PLLMULL6);

    // Enable GPIO clock
    GPIO::EnableClock(RCC_AHBENR_GPIOAEN | RCC_AHBENR_GPIOBEN);

    // Configure USART1
    USART usart(GPIO_Pin_2 | GPIO_Pin_3);
    usart.Config(115200);
    usart.Write("\r\nHello Inverter World\r\n");

    // Configure GPIOA pins for alternate function: PA6:TIM3_CH1
    GPIO gpioa6(GPIOA, GPIO_Pin_6, GPIO_AF, GPIO_PULL_UP);
    gpioa6.ConfigAF(GPIO_AF_1);

    // Configure GPIOA pins for alternate function: PA10:TIM1_CH3
    GPIO gpioa10(GPIOA, GPIO_Pin_10, GPIO_AF, GPIO_PULL_UP);
    gpioa10.ConfigAF(GPIO_AF_2);

    // Configure GPIOB pins for alternate function: PB1:TIM1_CH3N
    GPIO gpiob1(GPIOB, GPIO_Pin_1, GPIO_AF, GPIO_PULL_UP);
    gpiob1.ConfigAF(GPIO_AF_2);

    // Configure TIM1; CH3 - 20kHz; PWM mode; Master mode: generates 20kHz clock to TIM14
    RCC->APB2ENR |= RCC_APB2ENR_TIM1EN;             // enable TIM1 peripheral clock
    TIM1->PSC = 0;                                  // prescale TIM1 to run at SystemCoreClock (48MHz)
    TIM1->CCMR2 |= OC_MODE_PWM1 | TIM_CCMR2_OC3PE;  // set CH3 PWM mode, enable preload
    TIM1->ARR = (SystemCoreClock / 20000) - 1;            // auto-reload value for period 20kHz
    TIM1->CCR3 = TIM1->ARR >> 1;                    // initial 50% duty
    TIM1->CCER |= TIM_CCER_CC3E | TIM_CCER_CC3NE;   // enable CH3/CH3N
    TIM1->BDTR |= TIM_BDTR_MOE;                     // enable OC/OCN outputs
    TIM1->CR2 |= TIM_CR2_MMS_1;                     // set master mode: MMS=2

    // Configure TIM3; CH1 - 50Hz; PWM mode; Slave mode: clocked by TIM1 at 20kHz
    RCC->APB1ENR |= RCC_APB1ENR_TIM3EN;            // enable TIM3 peripheral clock
    TIM3->SMCR |= TIM_SMCR_SMS_2 | TIM_SMCR_SMS_1 | TIM_SMCR_SMS_0; // enable slave mode
    TIM3->PSC = 19;                                 // prescale TIM3 to run at 1ms
    TIM3->CCMR1 |= OC_MODE_PWM1 | TIM_CCMR1_OC1PE;  // set CH1 PWM mode, enable preload
    TIM3->ARR = 19;                                 // auto-reload value for period 50Hz
    TIM3->CCR1 = 10;
    TIM3->CCER |= TIM_CCER_CC1E;                    // enable CH1

    TIM1->CR1 |= TIM_CR1_CEN;                       // enable timers
    TIM3->CR1 |= TIM_CR1_CEN;                       // enable timers

    while (1);
}
